@@ -25,6 +25,12 @@
  * Shows HTTPS usage with client certs and optional ssl engine use.
  * </DESC>
  */
+#ifdef _MSC_VER
+#ifndef _CRT_SECURE_NO_WARNINGS
+#define _CRT_SECURE_NO_WARNINGS  /* for fopen() */
+#endif
+#endif
+
 #include <stdio.h>
 
 #include <curl/curl.h>
@@ -49,7 +55,7 @@
 
 int main(void)
 {
-  CURL *curl;
+  CURL *curl = NULL;
   CURLcode res;
   FILE *headerfile;
   const char *pPassphrase = NULL;
@@ -61,94 +67,89 @@ int main(void)
   const char *pKeyName;
   const char *pKeyType;
 
-  const char *pEngine;
-
 #ifdef USE_ENGINE
   pKeyName  = "rsa_test";
   pKeyType  = "ENG";
-  pEngine   = "chil";            /* for nChiper HSM... */
 #else
   pKeyName  = "testkey.pem";
   pKeyType  = "PEM";
-  pEngine   = NULL;
 #endif
 
+  res = curl_global_init(CURL_GLOBAL_ALL);
+  if(res) {
+    return (int)res;
+  }
+
   headerfile = fopen(pHeaderFile, "wb");
   if(!headerfile)
-    return 1;
-
-  curl_global_init(CURL_GLOBAL_DEFAULT);
+    goto error;
 
   curl = curl_easy_init();
-  if(curl) {
-    /* what call to write: */
-    curl_easy_setopt(curl, CURLOPT_URL, "HTTPS://your.favourite.ssl.site");
-    curl_easy_setopt(curl, CURLOPT_HEADERDATA, headerfile);
+  if(!curl)
+    goto error;
 
-#ifdef _MSC_VER
-#pragma warning(push)
-#pragma warning(disable:4127)  /* conditional expression is constant */
-#endif
-    do { /* dummy loop, just to break out from */
-      if(pEngine) {
-        /* use crypto engine */
-        if(curl_easy_setopt(curl, CURLOPT_SSLENGINE, pEngine) != CURLE_OK) {
-          /* load the crypto engine */
-          fprintf(stderr, "cannot set crypto engine\n");
-          break;
-        }
-        if(curl_easy_setopt(curl, CURLOPT_SSLENGINE_DEFAULT, 1L) != CURLE_OK) {
-          /* set the crypto engine as default */
-          /* only needed for the first time you load
-             an engine in a curl object... */
-          fprintf(stderr, "cannot set crypto engine as default\n");
-          break;
-        }
-      }
-      /* cert is stored PEM coded in file... */
-      /* since PEM is default, we needn't set it for PEM */
-      curl_easy_setopt(curl, CURLOPT_SSLCERTTYPE, "PEM");
+  /* what call to write: */
+  curl_easy_setopt(curl, CURLOPT_URL, "HTTPS://secure.site.example");
+  curl_easy_setopt(curl, CURLOPT_HEADERDATA, headerfile);
 
-      /* set the cert for client authentication */
-      curl_easy_setopt(curl, CURLOPT_SSLCERT, pCertFile);
-
-      /* sorry, for engine we must set the passphrase
-         (if the key has one...) */
-      if(pPassphrase)
-        curl_easy_setopt(curl, CURLOPT_KEYPASSWD, pPassphrase);
-
-      /* if we use a key stored in a crypto engine,
-         we must set the key type to "ENG" */
-      curl_easy_setopt(curl, CURLOPT_SSLKEYTYPE, pKeyType);
-
-      /* set the private key (file or ID in engine) */
-      curl_easy_setopt(curl, CURLOPT_SSLKEY, pKeyName);
-
-      /* set the file with the certs validating the server */
-      curl_easy_setopt(curl, CURLOPT_CAINFO, pCACertFile);
-
-      /* disconnect if we cannot validate server's cert */
-      curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 1L);
-
-      /* Perform the request, res gets the return code */
-      res = curl_easy_perform(curl);
-      /* Check for errors */
-      if(res != CURLE_OK)
-        fprintf(stderr, "curl_easy_perform() failed: %s\n",
-                curl_easy_strerror(res));
-
-      /* we are done... */
-    } while(0);
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
-    /* always cleanup */
-    curl_easy_cleanup(curl);
+#ifdef USE_ENGINE
+  /* use crypto engine. nCipher HSM... */
+  if(curl_easy_setopt(curl, CURLOPT_SSLENGINE, "chil") != CURLE_OK) {
+    /* load the crypto engine */
+    fprintf(stderr, "cannot set crypto engine\n");
+    goto error;
   }
+  if(curl_easy_setopt(curl, CURLOPT_SSLENGINE_DEFAULT, 1L) != CURLE_OK) {
+    /* set the crypto engine as default */
+    /* only needed for the first time you load
+       an engine in a curl object... */
+    fprintf(stderr, "cannot set crypto engine as default\n");
+    goto error;
+  }
+#endif
+
+  /* cert is stored PEM coded in file... */
+  /* since PEM is default, we need not set it for PEM */
+  curl_easy_setopt(curl, CURLOPT_SSLCERTTYPE, "PEM");
+
+  /* set the cert for client authentication */
+  curl_easy_setopt(curl, CURLOPT_SSLCERT, pCertFile);
+
+  /* sorry, for engine we must set the passphrase
+     (if the key has one...) */
+  if(pPassphrase)
+    curl_easy_setopt(curl, CURLOPT_KEYPASSWD, pPassphrase);
+
+  /* if we use a key stored in a crypto engine,
+     we must set the key type to "ENG" */
+  curl_easy_setopt(curl, CURLOPT_SSLKEYTYPE, pKeyType);
+
+  /* set the private key (file or ID in engine) */
+  curl_easy_setopt(curl, CURLOPT_SSLKEY, pKeyName);
+
+  /* set the file with the certs validating the server */
+  curl_easy_setopt(curl, CURLOPT_CAINFO, pCACertFile);
+
+  /* disconnect if we cannot validate server's cert */
+  curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 1L);
+
+  /* Perform the request, res gets the return code */
+  res = curl_easy_perform(curl);
+  /* Check for errors */
+  if(res != CURLE_OK)
+    fprintf(stderr, "curl_easy_perform() failed: %s\n",
+            curl_easy_strerror(res));
+
+error:
+
+  /* always cleanup */
+  if(curl)
+    curl_easy_cleanup(curl);
+
+  if(headerfile)
+    fclose(headerfile);
 
   curl_global_cleanup();
 
-  fclose(headerfile);
-
-  return 0;
+  return (int)res;
 }
