@@ -21,18 +21,29 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
+
 #include "curl_setup.h"
 
+#include <curl/curl.h>
+
 #include "urldata.h"
 #include "url.h"
 #include "cfilters.h"
 #include "curl_trc.h"
 #include "multiif.h"
+#include "curlx/timeval.h"
 #include "multi_ev.h"
 #include "select.h"
 #include "uint-bset.h"
 #include "uint-spbset.h"
+#include "uint-table.h"
+#include "curlx/warnless.h"
 #include "multihandle.h"
+#include "socks.h"
+
+/* The last 2 #include files should be in this order */
+#include "curl_memory.h"
+#include "memdebug.h"
 
 
 static void mev_in_callback(struct Curl_multi *multi, bool value)
@@ -44,7 +55,7 @@ static void mev_in_callback(struct Curl_multi *multi, bool value)
  * what to supervise (CURL_POLL_IN/CURL_POLL_OUT/CURL_POLL_REMOVE)
  */
 struct mev_sh_entry {
-  struct uint32_spbset xfers; /* bitset of transfers `mid`s on this socket */
+  struct uint_spbset xfers; /* bitset of transfers `mid`s on this socket */
   struct connectdata *conn; /* connection using this socket or NULL */
   void *user_data;      /* libcurl app data via curl_multi_assign() */
   unsigned int action;  /* CURL_POLL_IN/CURL_POLL_OUT we last told the
@@ -57,7 +68,7 @@ struct mev_sh_entry {
 
 static size_t mev_sh_entry_hash(void *key, size_t key_length, size_t slots_num)
 {
-  curl_socket_t fd = *((curl_socket_t *)key);
+  curl_socket_t fd = *((curl_socket_t *) key);
   (void)key_length;
   return (fd % (curl_socket_t)slots_num);
 }
@@ -65,22 +76,21 @@ static size_t mev_sh_entry_hash(void *key, size_t key_length, size_t slots_num)
 static size_t mev_sh_entry_compare(void *k1, size_t k1_len,
                                    void *k2, size_t k2_len)
 {
-  (void)k1_len;
-  (void)k2_len;
-  return (*((curl_socket_t *)k1)) == (*((curl_socket_t *)k2));
+  (void)k1_len; (void)k2_len;
+  return (*((curl_socket_t *) k1)) == (*((curl_socket_t *) k2));
 }
 
 /* sockhash entry destructor callback */
 static void mev_sh_entry_dtor(void *freethis)
 {
   struct mev_sh_entry *entry = (struct mev_sh_entry *)freethis;
-  Curl_uint32_spbset_destroy(&entry->xfers);
-  curlx_free(entry);
+  Curl_uint_spbset_destroy(&entry->xfers);
+  free(entry);
 }
 
 /* look up a given socket in the socket hash, skip invalid sockets */
-static struct mev_sh_entry *mev_sh_entry_get(struct Curl_hash *sh,
-                                             curl_socket_t s)
+static struct mev_sh_entry *
+mev_sh_entry_get(struct Curl_hash *sh, curl_socket_t s)
 {
   if(s != CURL_SOCKET_BAD) {
     /* only look for proper sockets */
@@ -90,8 +100,8 @@ static struct mev_sh_entry *mev_sh_entry_get(struct Curl_hash *sh,
 }
 
 /* make sure this socket is present in the hash for this handle */
-static struct mev_sh_entry *mev_sh_entry_add(struct Curl_hash *sh,
-                                             curl_socket_t s)
+static struct mev_sh_entry *
+mev_sh_entry_add(struct Curl_hash *sh, curl_socket_t s)
 {
   struct mev_sh_entry *there = mev_sh_entry_get(sh, s);
   struct mev_sh_entry *check;
@@ -102,11 +112,11 @@ static struct mev_sh_entry *mev_sh_entry_add(struct Curl_hash *sh,
   }
 
   /* not present, add it */
-  check = curlx_calloc(1, sizeof(struct mev_sh_entry));
+  check = calloc(1, sizeof(struct mev_sh_entry));
   if(!check)
     return NULL; /* major failure */
 
-  Curl_uint32_spbset_init(&check->xfers);
+  Curl_uint_spbset_init(&check->xfers);
 
   /* make/add new hash entry */
   if(!Curl_hash_add(sh, (char *)&s, sizeof(curl_socket_t), check)) {
@@ -125,13 +135,13 @@ static void mev_sh_entry_kill(struct Curl_multi *multi, curl_socket_t s)
 
 static size_t mev_sh_entry_user_count(struct mev_sh_entry *e)
 {
-  return Curl_uint32_spbset_count(&e->xfers) + (e->conn ? 1 : 0);
+  return Curl_uint_spbset_count(&e->xfers) + (e->conn ? 1 : 0);
 }
 
 static bool mev_sh_entry_xfer_known(struct mev_sh_entry *e,
                                     struct Curl_easy *data)
 {
-  return Curl_uint32_spbset_contains(&e->xfers, data->mid);
+  return Curl_uint_spbset_contains(&e->xfers, data->mid);
 }
 
 static bool mev_sh_entry_conn_known(struct mev_sh_entry *e,
@@ -145,7 +155,7 @@ static bool mev_sh_entry_xfer_add(struct mev_sh_entry *e,
 {
    /* detect weird values */
   DEBUGASSERT(mev_sh_entry_user_count(e) < 100000);
-  return Curl_uint32_spbset_add(&e->xfers, data->mid);
+  return Curl_uint_spbset_add(&e->xfers, data->mid);
 }
 
 static bool mev_sh_entry_conn_add(struct mev_sh_entry *e,
@@ -160,12 +170,13 @@ static bool mev_sh_entry_conn_add(struct mev_sh_entry *e,
   return TRUE;
 }
 
+
 static bool mev_sh_entry_xfer_remove(struct mev_sh_entry *e,
                                      struct Curl_easy *data)
 {
-  bool present = Curl_uint32_spbset_contains(&e->xfers, data->mid);
+  bool present = Curl_uint_spbset_contains(&e->xfers, data->mid);
   if(present)
-    Curl_uint32_spbset_remove(&e->xfers, data->mid);
+    Curl_uint_spbset_remove(&e->xfers, data->mid);
   return present;
 }
 
@@ -195,7 +206,8 @@ static CURLMcode mev_forget_socket(struct Curl_multi *multi,
 
   /* We managed this socket before, tell the socket callback to forget it. */
   if(entry->announced && multi->socket_cb) {
-    CURL_TRC_M(data, "ev %s, call(fd=%" FMT_SOCKET_T ", ev=REMOVE)", cause, s);
+    CURL_TRC_M(data, "ev %s, call(fd=%" FMT_SOCKET_T ", ev=REMOVE)",
+               cause, s);
     mev_in_callback(multi, TRUE);
     rc = multi->socket_cb(data, s, CURL_POLL_REMOVE,
                           multi->socket_userp, entry->user_data);
@@ -344,7 +356,7 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
                  ", total=%u/%d (xfer/conn)", s,
                  conn ? "connection" : "transfer",
                  conn ? conn->connection_id : data->mid,
-                 Curl_uint32_spbset_count(&entry->xfers),
+                 Curl_uint_spbset_count(&entry->xfers),
                  entry->conn ? 1 : 0);
     }
     else {
@@ -412,7 +424,7 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
         return mresult;
       CURL_TRC_M(data, "ev entry fd=%" FMT_SOCKET_T ", removed transfer, "
                  "total=%u/%d (xfer/conn)", s,
-                 Curl_uint32_spbset_count(&entry->xfers),
+                 Curl_uint_spbset_count(&entry->xfers),
                  entry->conn ? 1 : 0);
     }
     else {
@@ -434,11 +446,12 @@ static void mev_pollset_dtor(void *key, size_t klen, void *entry)
   (void)klen;
   if(ps) {
     Curl_pollset_cleanup(ps);
-    curlx_free(ps);
+    free(ps);
   }
 }
 
-static struct easy_pollset *mev_add_new_conn_pollset(struct connectdata *conn)
+static struct easy_pollset*
+mev_add_new_conn_pollset(struct connectdata *conn)
 {
   struct easy_pollset *ps;
 
@@ -450,7 +463,8 @@ static struct easy_pollset *mev_add_new_conn_pollset(struct connectdata *conn)
   return ps;
 }
 
-static struct easy_pollset *mev_add_new_xfer_pollset(struct Curl_easy *data)
+static struct easy_pollset*
+mev_add_new_xfer_pollset(struct Curl_easy *data)
 {
   struct easy_pollset *ps;
 
@@ -462,8 +476,9 @@ static struct easy_pollset *mev_add_new_xfer_pollset(struct Curl_easy *data)
   return ps;
 }
 
-static struct easy_pollset *mev_get_last_pollset(struct Curl_easy *data,
-                                                 struct connectdata *conn)
+static struct easy_pollset *
+mev_get_last_pollset(struct Curl_easy *data,
+                     struct connectdata *conn)
 {
   if(data) {
     if(conn)
@@ -478,7 +493,7 @@ static CURLMcode mev_assess(struct Curl_multi *multi,
                             struct connectdata *conn)
 {
   struct easy_pollset ps, *last_ps;
-  CURLMcode mresult = CURLM_OK;
+  CURLMcode res = CURLM_OK;
 
   if(!multi || !multi->socket_cb)
     return CURLM_OK;
@@ -487,8 +502,8 @@ static CURLMcode mev_assess(struct Curl_multi *multi,
   if(conn) {
     CURLcode r = Curl_conn_adjust_pollset(data, conn, &ps);
     if(r) {
-      mresult = (r == CURLE_OUT_OF_MEMORY) ?
-        CURLM_OUT_OF_MEMORY : CURLM_INTERNAL_ERROR;
+      res = (r == CURLE_OUT_OF_MEMORY) ?
+            CURLM_OUT_OF_MEMORY : CURLM_INTERNAL_ERROR;
       goto out;
     }
   }
@@ -502,18 +517,18 @@ static CURLMcode mev_assess(struct Curl_multi *multi,
     else
       last_ps = mev_add_new_xfer_pollset(data);
     if(!last_ps) {
-      mresult = CURLM_OUT_OF_MEMORY;
+      res = CURLM_OUT_OF_MEMORY;
       goto out;
     }
   }
 
   if(last_ps)
-    mresult = mev_pollset_diff(multi, data, conn, &ps, last_ps);
+    res = mev_pollset_diff(multi, data, conn, &ps, last_ps);
   else
     DEBUGASSERT(!ps.n);
 out:
   Curl_pollset_cleanup(&ps);
-  return mresult;
+  return res;
 }
 
 CURLMcode Curl_multi_ev_assess_xfer(struct Curl_multi *multi,
@@ -530,22 +545,23 @@ CURLMcode Curl_multi_ev_assess_conn(struct Curl_multi *multi,
 }
 
 CURLMcode Curl_multi_ev_assess_xfer_bset(struct Curl_multi *multi,
-                                         struct uint32_bset *set)
+                                         struct uint_bset *set)
 {
-  uint32_t mid;
-  CURLMcode mresult = CURLM_OK;
+  unsigned int mid;
+  CURLMcode result = CURLM_OK;
 
-  if(multi && multi->socket_cb && Curl_uint32_bset_first(set, &mid)) {
+  if(multi && multi->socket_cb && Curl_uint_bset_first(set, &mid)) {
     do {
       struct Curl_easy *data = Curl_multi_get_easy(multi, mid);
-      if(data) {
-        mresult = Curl_multi_ev_assess_xfer(multi, data);
-      }
-    } while(!mresult && Curl_uint32_bset_next(set, mid, &mid));
+      if(data)
+        result = Curl_multi_ev_assess_xfer(multi, data);
+    }
+    while(!result && Curl_uint_bset_next(set, mid, &mid));
   }
-  return mresult;
+  return result;
 }
 
+
 CURLMcode Curl_multi_ev_assign(struct Curl_multi *multi,
                                curl_socket_t s,
                                void *user_data)
@@ -558,7 +574,8 @@ CURLMcode Curl_multi_ev_assign(struct Curl_multi *multi,
 }
 
 void Curl_multi_ev_dirty_xfers(struct Curl_multi *multi,
-                               curl_socket_t s)
+                               curl_socket_t s,
+                               bool *run_cpool)
 {
   struct mev_sh_entry *entry;
 
@@ -572,9 +589,9 @@ void Curl_multi_ev_dirty_xfers(struct Curl_multi *multi,
      and just move on. */
   if(entry) {
     struct Curl_easy *data;
-    uint32_t mid;
+    unsigned int mid;
 
-    if(Curl_uint32_spbset_first(&entry->xfers, &mid)) {
+    if(Curl_uint_spbset_first(&entry->xfers, &mid)) {
       do {
         data = Curl_multi_get_easy(multi, mid);
         if(data) {
@@ -582,13 +599,14 @@ void Curl_multi_ev_dirty_xfers(struct Curl_multi *multi,
         }
         else {
           CURL_TRC_M(multi->admin, "socket transfer %u no longer found", mid);
-          Curl_uint32_spbset_remove(&entry->xfers, mid);
+          Curl_uint_spbset_remove(&entry->xfers, mid);
         }
-      } while(Curl_uint32_spbset_next(&entry->xfers, mid, &mid));
+      }
+      while(Curl_uint_spbset_next(&entry->xfers, mid, &mid));
     }
 
     if(entry->conn)
-      Curl_multi_mark_dirty(multi->admin);
+      *run_cpool = TRUE;
   }
 }
 
