@@ -235,6 +235,13 @@ static const char *disabled[]={
   "ON"
 #else
   "OFF"
+#endif
+  ,
+  "ssl-sessions: "
+#ifdef USE_SSLS_EXPORT
+  "ON"
+#else
+  "OFF"
 #endif
 };
 
@@ -246,6 +253,7 @@ int main(int argc, char **argv)
   (void)argv;
 
   for(i = 0; i < CURL_ARRAYSIZE(disabled); i++)
+    /* !checksrc! disable BANNEDFUNC 1 */
     printf("%s\n", disabled[i]);
 
   return 0;
